@@ -59,7 +59,7 @@ tp_f_get_qint16 (decoder_qdatastream_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -84,7 +84,7 @@ tp_f_get_qint32 (decoder_qdatastream_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -109,7 +109,7 @@ tp_f_get_qint64 (decoder_qdatastream_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -134,7 +134,7 @@ tp_f_get_qstring (decoder_qdatastream_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -159,7 +159,7 @@ tp_f_get_qbytearray (decoder_qdatastream_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -184,7 +184,7 @@ tp_f_get_qdatetime (decoder_qdatastream_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -209,7 +209,7 @@ tp_f_get_qimage (decoder_qdatastream_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -234,7 +234,7 @@ tp_f_get_qpixmap (decoder_qdatastream_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -318,7 +318,7 @@ tp_new (PyTypeObject *type, PyObject *args, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -334,7 +334,7 @@ tp_new (PyTypeObject *type, PyObject *args, PyObject *)
       catch (const std::exception& e)
         {
           Py_DECREF (ret);
-          PyErr_SetString (PyExc_Exception, e.what ());
+          mobius::py::set_runtime_error (e.what ());
           ret = nullptr;
         }
     }
